'use strict';

/**
 * 请求参数校验
 * @return {function} The parameter validate function
 */
module.exports = () => {

  return async function validation(ctx, next) {

    // 取得LOGGER
    const { loggerName } = ctx.app.config.validation;
    const LOGGER = loggerName === '' ? ctx.logger : ctx.getLogger(loggerName);
    LOGGER.info('[egg-validation] 处理开始');

    // 校验参数
    if (!validate(ctx, LOGGER)) {
      const { validation } = ctx.app.config;
      ctx.body = { code: validation.errorCode || 400 };
    } else {
      LOGGER.info('[egg-validation] 参数校验通过');
      await next();
    }
  };
};

/**
 * 校验参数
 * @param {object} ctx Application context
 * @param {object} LOGGER Logger function
 * @return {boolean} Validation result
 */
function validate(ctx, LOGGER) {

  LOGGER.debug('[egg-validation] 开始校验参数');

  try {

    // 取得配置参数
    const { rules, whitelist, injectParams } = ctx.app.config.validation;

    // 取得路径参数
    const { stack } = ctx.app.router;
    const { method, url } = ctx.request;

    // 计算当前匹配的URL
    const currentRoute = stack.find(s => {
      return url.indexOf('?') > 0 ?
        (new RegExp(s.regexp)).test(url.replace(/\b\?\S*/, '')) :
        (new RegExp(s.regexp)).test(url);
    });
    const path = currentRoute ? (method + ' ' + currentRoute.path) : '未匹配到路径';
    LOGGER.debug('[egg-validation] 匹配URL= ' + path);

    // 取得查询或提交的参数
    const params = method === 'GET' ? ctx.query : ctx.request.body;

    // 取得校验规则并开始校验
    if (whitelist.indexOf(path) >= 0) {
      LOGGER.debug('[egg-validation] 该URL在白名单中，自动校验通过');
    } else {

      // 取得校验规则
      const RULE = rules[path];
      LOGGER.debug('[egg-validation] 取得校验规则', RULE);

      // 如果校验未通过，返回false
      if (ctx.validate(RULE, params)) return false;
    }

    // 根据需要，将params设置到ctx
    LOGGER.debug('[egg-validation] 是否设置参数？' + (injectParams ? '是' : '否'));
    if (injectParams && params) {
      ctx.params = ctx.params || {};
      ctx.params = Object.assign(ctx.params, params);
    }

    return true;

  } catch (err) {
    LOGGER.error('[egg-validation] 参数校验未通过');
    LOGGER.error('[egg-validation] 参数校验错误内容=', err);
    return false;
  }
}
